/*--------------------------------------------------------
LCD-Routine fr LCDSER V2 und 
HD44780 im 4Bit Modus V1.0
Dipl.-Ing. Walter  Steiner (12.01.2012)
----------------------------------------------------------
Korrektur: 
----------------------------------------------------------

Fr die Ansteuerung des Displays werden 7 Pins bentigt.
Die Busy Leitung wird nicht verwendet.

Pinbelegung:
LCDSER
PORT Pin lcd_rs     PD2
PORT Pin lcd_rw     PD3
PORT Pin lcd_e      PD4
PORT Pin 4          PB4
PORT Pin 5          PB5
PORT Pin 6          PB6
PORT Pin 7          PB7

LCDUSB: RS und E sind vertauscht
PORT Pin lcd_rs     PD4
PORT Pin lcd_rw     PD3
PORT Pin lcd_e      PD2
PORT Pin 4          PB4
PORT Pin 5          PB5
PORT Pin 6          PB6
PORT Pin 7          PB7

*/


#define F_CPU 1000000UL

// Includes
#include <avr/io.h>
#include <util/delay.h>

#include "util.h"


// Funktionsprototypen
void lcd_data(char byte);
void lcd_command(char byte);
void lcd_enable(void);
void kpause(void);
void lpause(void);


//LCDDA
#define lcd_e  6    //E Pinnummer PORTD
#define lcd_rw 5    //RW Pinnummer PORTD
#define lcd_rs 4    //RS Pinnummer PORTD

#define CONTRPORT PORTC
#define DATAPORT PORTC


// Makros
#define setBit(adr,bit) (adr|=(1<<bit))
#define clrBit(adr,bit) (adr&= ~(1<<bit))
#define invBit(adr,bit) (adr^= (1<<bit))
#define isBit(adr,bit)  (adr&(1<<bit))

int p=0;

// sendet ein Datenbyte an das LCD
void lcd_data(char byte)
{
    setBit(CONTRPORT, lcd_rs);

    DATAPORT &= 0xF0;
    DATAPORT |= (byte&0xF0)>>4;
    lcd_enable();
	
    DATAPORT &= 0xF0;
    DATAPORT |= byte&0x0f;
    lcd_enable();

    kpause();
}


//sendet einen Befehl an das LCD
void lcd_command(char byte)
{
    clrBit(CONTRPORT, lcd_rs);

    DATAPORT &= 0xF0;
    DATAPORT |= (byte&0xF0)>>4;
    lcd_enable();
	
    DATAPORT &= 0xF0;
    DATAPORT |= byte&0x0F;
    lcd_enable();

    kpause();
}


//erzeugt den Enable-Puls
void lcd_enable(void)
{
    setBit(CONTRPORT, lcd_e);
    kpause();
    clrBit(CONTRPORT, lcd_e);
}                 


//Pause nach jeder bertragung
void kpause(void)
{
    _delay_us(48);
//    _delay_us(10);
}


//Lngere Pause fr manche Befehle
void lpause(void) 
{
    _delay_ms(5);
}


//Initialisierung
void initLcd(void)
{

    DDRC |= 0b01111111;         //Pins 0-6 als Ausgnge 
    clrBit(CONTRPORT, lcd_rw);

// =============== RESET ===============
    DATAPORT &= 0xF0;           // Bit 0..3 = 0
    clrBit(CONTRPORT,lcd_rs);
    clrBit(CONTRPORT,lcd_e);
    _delay_ms(20);
    setBit(CONTRPORT,lcd_e);
// =============== RESET ===============

    DATAPORT |= 0b00000010;     //Interface auf 4-Bit setzen
    lcd_enable();
    lpause();

    lcd_command(0b00101000);    // 2-zeilig, 5x7-Punkt-Matrix
    lcd_command(0b00001100);    // Display ein
    lcd_command(0b00000110);    // cursor right, no disp. shift
}


// loescht das Display
void blankLcd(void)
{
    lcd_command(0x01);
    lpause();
}


// gibt ein Zeichen aus
void charLcd(char zeile, char spalte, char ch)
{
    char adr;

    if(zeile<1 || zeile>2 || spalte<1 || spalte>20)
        return;

    adr = (zeile-1)*0x40 + spalte-1;
    lcd_command(adr|0x80);
    lcd_data(ch);
}


// gibt einen String aus
void printLcd(char zeile, char spalte, char* str)
{
    char adr;

    if(zeile<1 || zeile>2 || spalte<1 || spalte>20)
        return;

    adr = (zeile-1)*0x40 + spalte-1;
    lcd_command(adr|0x80);

    while (*str)
    {
        lcd_data(*str++);
    }
}


